import { Merchant } from './Merchant';
import { BasePaymentParams, CreditOneTimePaymentParams, CreditDividePaymentParams, CreditPeriodPaymentParams, WebATMPaymentParams, ATMPaymentParams, CVSPaymentParams, BARCODEPaymentParams, AndroidPayPaymentParams, ALLPaymentParams, InvoiceParams, ECPayPaymentType } from '../types';
export declare class Payment<T> {
    merchant: Merchant;
    apiUrl: string;
    baseParams: BasePaymentParams & {
        EncryptType: 1;
        ChoosePayment: ECPayPaymentType;
        InvoiceMark: 'N' | 'Y';
        PaymentType: 'aio';
    };
    params: T;
    invoice?: InvoiceParams;
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: T);
    checkout(invoice?: InvoiceParams): Promise<string>;
}
export declare class CreditOneTimePayment extends Payment<CreditOneTimePaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: CreditOneTimePaymentParams);
}
export declare class CreditDividePayment extends Payment<CreditDividePaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: CreditDividePaymentParams);
}
export declare class CreditPeriodPayment extends Payment<CreditPeriodPaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: CreditPeriodPaymentParams);
}
export declare class WebATMPayment extends Payment<WebATMPaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: WebATMPaymentParams);
}
export declare class ATMPayment extends Payment<ATMPaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: ATMPaymentParams);
}
export declare class CVSPayment extends Payment<CVSPaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: CVSPaymentParams);
}
export declare class BARCODEPayment extends Payment<BARCODEPaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: BARCODEPaymentParams);
}
export declare class AndroidPayPayment extends Payment<AndroidPayPaymentParams> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: AndroidPayPaymentParams);
}
export declare class ALLPayment extends Payment<Partial<ALLPaymentParams>> {
    constructor(merchant: Merchant, baseParams: BasePaymentParams, params: Partial<ALLPaymentParams>);
}
