"use strict";
var __awaiter = (this && this.__awaiter) || function (thisArg, _arguments, P, generator) {
    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
    return new (P || (P = Promise))(function (resolve, reject) {
        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }
        function rejected(value) { try { step(generator["throw"](value)); } catch (e) { reject(e); } }
        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
        step((generator = generator.apply(thisArg, _arguments || [])).next());
    });
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.parseIntegerFileds = exports.fromEntries = exports.getQueryStringFromParams = exports.PostRequest = exports.getDateTimeString = exports.getDateString = exports.getCurrentUnixTimeStampOffset = exports.getEncodedInvoice = exports.generateRedirectPostForm = exports.generateCheckMacValue = void 0;
const crypto_1 = require("crypto");
const url_1 = require("url");
const buffer_1 = require("buffer");
const https_1 = require("https");
const iconv_lite_1 = require("iconv-lite");
function generateCheckMacValue(params, hashKey, hashIV) {
    const _params = Object.assign({}, params);
    const excludedParams = ['HashKey', 'HashIV', 'CheckMacValue'];
    excludedParams.forEach((p) => delete _params[p]);
    const mac = Object.keys(_params)
        .sort((a, b) => {
        if (a.toUpperCase() < b.toUpperCase())
            return -1;
        if (a.toUpperCase() > b.toUpperCase())
            return 1;
        return 0;
    })
        .reduce((prev, curr) => (prev += `${curr}=${_params[curr]}&`), '');
    // TBD: 手冊 p.27 CarruerNum 條目說明待驗證
    //      =>  若手機條碼有 + 號, 先轉成空白再產生驗證碼(會在.net URLEncode的 +)
    // if (_params.CarruerType && _params.CarruerType === '3') {
    //   _params.CarruerNum.replace(/+/g, ' ');
    // }
    // API文件: 附錄四 URLEncode 轉換表, .net URLEncode
    // src_string => (step-1) encodeURIComponent
    //            => (step-2) turn certain chars back => .net URLEncoded string
    const urlParamsStr = encodeURIComponent(`HashKey=${hashKey}&${mac}HashIV=${hashIV}`)
        .toLowerCase()
        .replace(/%2d/g, '-')
        .replace(/%5f/g, '_')
        .replace(/%2e/g, '.')
        .replace(/%21/g, '!')
        .replace(/%2a/g, '*')
        .replace(/%28/g, '(')
        .replace(/%29/g, ')')
        .replace(/%20/g, '+');
    return (0, crypto_1.createHash)('sha256').update(urlParamsStr).digest('hex').toUpperCase();
}
exports.generateCheckMacValue = generateCheckMacValue;
function generateRedirectPostForm(endpoint, params) {
    const formId = '_form_aio_checkout';
    const inputs = Object.keys(params)
        .sort() // ensure every test to have the form with key=value pairs in the same order
        .reduce((prev, curr) => {
        let currVal = params[curr];
        // ignore undefined fileds
        return currVal === undefined
            ? prev
            : (prev += `<input type="hidden" name="${curr}" id="${curr}" value="${currVal}" />`);
    }, '') +
        `<script type="text/javascript">document.getElementById("${formId}").submit();</script>`;
    return `<form id="${formId}" action="${endpoint}" method="post">${inputs}</form>`;
}
exports.generateRedirectPostForm = generateRedirectPostForm;
function getEncodedInvoice(invoice) {
    if (!invoice)
        return;
    const _invoice = Object.assign({}, invoice);
    // remove any 'key=undefined' pair before encodeURIComponent()
    Object.keys(_invoice).forEach((k) => {
        if (_invoice[k] === undefined)
            delete _invoice[k];
    });
    const encodingFileds = [
        'CustomerName',
        'CustomerAddr',
        'CustomerEmail',
        'InvoiceItemName',
        'InvoiceItemWord',
        'InvoiceRemark',
    ];
    encodingFileds.forEach((key) => {
        let val = _invoice[key];
        if (val)
            _invoice[key] = encodeURIComponent(val);
    });
    return _invoice;
}
exports.getEncodedInvoice = getEncodedInvoice;
function getCurrentUnixTimeStampOffset(seconds) {
    seconds = seconds || 0;
    return Math.floor(new Date().getTime() / 1000) + seconds;
}
exports.getCurrentUnixTimeStampOffset = getCurrentUnixTimeStampOffset;
function getDateString(date) {
    date = date || new Date();
    return date.toLocaleDateString('zh-TW', {
        year: 'numeric',
        month: '2-digit',
        day: '2-digit',
    });
}
exports.getDateString = getDateString;
function getDateTimeString(date) {
    date = date || new Date();
    return `${getDateString(date)} ${date.toLocaleTimeString('zh-TW', {
        hour12: false,
        hour: '2-digit',
        minute: '2-digit',
        second: '2-digit',
    })}`;
}
exports.getDateTimeString = getDateTimeString;
function PostRequest(config) {
    return __awaiter(this, void 0, void 0, function* () {
        const { apiUrl, params, responseEncoding = 'utf8' } = config;
        const _url = new url_1.URL(apiUrl);
        const postData = getQueryStringFromParams(params, true);
        const options = {
            protocol: _url.protocol,
            hostname: _url.hostname,
            hash: _url.hash,
            search: _url.search,
            pathname: _url.pathname,
            path: `${_url.pathname}${_url.search}`,
            href: _url.toString(),
            method: 'POST',
            headers: {
                'Content-Type': 'application/x-www-form-urlencoded',
                'Content-Length': buffer_1.Buffer.byteLength(postData),
            },
        };
        return new Promise((resolve, reject) => {
            const req = (0, https_1.request)(options, (rsp) => {
                const decodedRsp = (0, iconv_lite_1.decodeStream)(responseEncoding);
                rsp.pipe(decodedRsp);
                let dataStr = '';
                // rsp.setEncoding('binary'); // default is binary
                decodedRsp.on('data', (chunk) => (dataStr += chunk));
                decodedRsp.on('end', () => {
                    try {
                        //if  QueryCreditCardPeriodInfo: parse json, otherwise parse x=y&m=n to object
                        let data;
                        if (apiUrl.endsWith('QueryCreditCardPeriodInfo')) {
                            // No Response CheckMacValue
                            data = JSON.parse(dataStr);
                        }
                        else if (
                        // No Response CheckMacValue
                        apiUrl.endsWith('TradeNoAio') ||
                            apiUrl.endsWith('FundingReconDetail')) {
                            data = dataStr;
                        }
                        else {
                            data = fromEntries(new url_1.URLSearchParams(dataStr));
                        }
                        resolve(data);
                    }
                    catch (err) {
                        reject(err);
                    }
                });
                decodedRsp.on('error', (err) => {
                    reject(err);
                });
            });
            req.on('error', (err) => {
                reject(err);
            });
            req.write(postData);
            req.end();
        });
    });
}
exports.PostRequest = PostRequest;
function getQueryStringFromParams(data, sort) {
    const _params = new url_1.URLSearchParams(data);
    // remove any 'key=undefined' pair
    Object.keys(data).forEach((k) => {
        if (data[k] === undefined)
            _params.delete(k);
    });
    if (sort)
        _params.sort();
    return _params.toString();
}
exports.getQueryStringFromParams = getQueryStringFromParams;
function fromEntries(entries) {
    return [...entries].reduce((obj, [key, val]) => {
        obj[key] = val;
        return obj;
    }, {});
}
exports.fromEntries = fromEntries;
function parseIntegerFileds(input, fields) {
    const result = Object.assign({}, input);
    fields.forEach((key) => {
        let val = result[key];
        if (typeof val === 'string' && val !== '') {
            let parsed = parseInt(val, 10);
            if (!isNaN(parsed))
                result[key] = parsed;
        }
    });
    return result;
}
exports.parseIntegerFileds = parseIntegerFileds;
// RtnCode, gwsr, amount, clsamt
// TradeAmt, HandlingCharge, PaymentTypeChargeFee
// PeriodAmount, TotalAmount, Frequency, ExecTimes,
// TotalSuccessTimes, TotalSuccessAmount
// RtnCode
